%% miu_model_3e_q12.m
%
%  Basic Money-in-the-utility function model from Chapter 2, 
%  Monetary theory and Policy, Third edition,
%  Carl E. Walsh
%  MIT Press, copyright
%
%  Used for problem 12 on the effects of b = 1, 2, 3, and 5;
%
%  See miu_dynamics.tex for more details
% 
%  If sol == 'SHU', solution method basic on Toolkit of Paul Soderlind
%  If sol == 'LRP', solution method basic on linear regular approach
% 
%  This program sets up the basic model in the form
%  A1*E_t(Z_t+1) = A2 * Z_t + V_t+1
%
%     E_t(Z_t+1) = inv(A1)*A2 * Z_t + inv(A1)*V_t+1
%
%     E_t(Z_t+1) = A * Z_t + PSI_t+1
%
%  Solution takes the form
%
%     Z1(t+1) = M*Z1(t) + shocks
%
%     Z2(t) = C*Z(t)
%
%  where Z1 is the vector of predetermined variables and Z2 is the vector
%  of forward-looking variables.
%
%  Actual solution calculated by basicsol.m if sol == 'SHU'
%  [M,C] = basicsol(A,n1,n2,SigmaXX,x10,cutoff);
%  or if sol == 'LRP' by
%  [M, G] = lrp(A,p.beta,n1,n2)
%
%  basicsol.m (or lrp.m) must be on path
%
%  n1 = number or predetermined variables
%  n2 = number of nonpredetermined (forward looking) variables
%  SigmaXX = (n1+n2) x (n1+n2) covariance matrix Cov[PSI(t)] of shocks.
%  x10 is an n1x1 vector, initial values for predetermined variables
%  cutoff = scalar, max modulus of backward looking variables (1.0)
% 
%% Housekeeping
clear all

% setting up values of b to employ
b = [1 2 3 5];

dispm = 0; % if = 1, matrices A1, A2, and A are displayed, if = 0, they are not displayed
sol = 'SHU'; % or 'SHU' or 'LRP'
imp = 1;  % set to 1 to calculate impulse resonses, 0 otherwise

%% Looping over values of b
ct = size(b,2);
Tbig = 24;  % length of impulse response

%% initiallizing matrices to store impulse responses
z_output_b = zeros(Tbig-1,ct);
z_n_b = zeros(Tbig-1,ct);
u_output_b = zeros(Tbig-1,ct);
u_n_b = zeros(Tbig-1,ct);

for ib=1:ct
%% Setting up model
n1 = 10; % z, u, y, c, k, x, n, r, m, i
n2 = 2; % lambda, pi

n = n1 + n2;
A1 = zeros(n,n);
A2 = zeros(n,n);

%% Parameters

% "Technology" parameters
p.N_bar = 1/3;    % steady state employment is a third of total time endowment
p.alpha = .36;    % Capital share
p.delta = .019;   % Depreciation rate for capital

%"Taste" parameters
p.beta = .989;    % discount parameters
p.PHI = 2;        % coefficient of relative risk aversion
p.eta = 1;        % leisure parameter

% Next two parameters are used for the MIU model but not for the CIA model

p.b = b(ib);      % interest elasticity
p.a = 0.99;       %

%% Exogenous stochastic processes

p.rho_z = .95;      % autocorrelation of techonology shock
p.sigma_z = .34; % standard deviation of techonology shock. Units: percent
    % 0.34 for PHI=2 and .48 for PHI = 8.

%% Exogenous policy specification
p.THETA = 1.01; % rate of money growth (gross)
p.rho_u = 0.5;    % autocorrelation of money shock
p.sigma_m = (.9/((1-(p.rho_u)^2)))^.5;% standard deviation of money growth rate shock. Units: Percent
p.phi = 0; %.15;    % correlation of techonology shock and money shock

% Innovation variance of money growth shock set so that s.d. of money
% growth matches p.sigma_m.
p.sigma_v = ((1-p.rho_u^2)*(p.sigma_m)^2 - ((p.phi^2)/(1-p.rho_z^2))*(p.sigma_z^2))^.5;

%% Calculating the steady state:
p.R_bar = 1/p.beta;                         % one percent real interest per quarter
p.YK_bar = (p.R_bar - 1 + p.delta)/p.alpha; % = Y_bar/K_bar
p.CK_bar = p.YK_bar - p.delta;              % = C_bar/K_bar
p.NK_bar = (p.YK_bar)^ (1/(1-p.alpha));     % = N_bar/K_bar
p.inf_bar = p.THETA - 1;                    % = inflation rate
p.NL_bar = p.N_bar/(1.0 - p.N_bar);         % = N_bar/L_bar
p.YN_bar = p.YK_bar /(1.0 - p.N_bar);       % = Y_bar/N_bar
p.I_bar = p.R_bar*p.THETA;                  % = (gross) nominal interest rate
p.i_bar = p.I_bar - 1;                      % = nominal interest rate

% Following used in MIU model but not in CIA model
p.mK_bar = (p.CK_bar) * (p.a*p.i_bar/((1-p.a) * (1+p.i_bar)))^ (-1/p.b); % = M_bar/K_bar
p.mC_bar = p.mK_bar /p.CK_bar;   % = m_bar/C_bar
p.H = (p.a*(p.CK_bar)^(1-p.b)) + (1-p.a)*(p.mK_bar^(1-p.b));
p.gamma = (p.a*(p.CK_bar)^(1-p.b))/p.H;

% Parameters defined in text
p.kappa = p.eta*p.N_bar/(1-p.N_bar); 
p.xi = p.alpha*(p.YK_bar)/(p.alpha + p.kappa + (1-p.alpha)*p.alpha*(p.YK_bar));
p.OMEGA1 = p.b*(p.gamma-1) - p.gamma*p.PHI;
p.OMEGA2 = (p.b - p.PHI)*(1-p.gamma);
p.theta = p.alpha + p.eta*p.NL_bar + (1-p.alpha)*p.alpha*p.YK_bar;


%% Model matrices
% equations for z(t+1), u(t+1), y(t), c(t), k(t), x(t) n(t), r(t), m(t), i(t)
% E_t(lambda(t+1)), E_t(pi(t+1))

% exogenous disturbances
A1(1,1) = 1;
A2(1,1) = p.rho_z;
A1(2,2) = 1;
A2(2,1) = p.phi;
A2(2,2) = p.rho_u;

% production function
A1(3,3) = 1;
A1(3,7) = p.alpha - 1;
A2(3,1) = 1;
A2(3,5) = p.alpha;

% resource constraint (goods market clearing)
A1(4,3) = p.YK_bar;
A1(4,4) = -p.CK_bar;
A1(4,6) = -p.delta;

% capital accumulation
A1(5,5) = 1;
A1(5,6) = -p.delta;
A2(5,5) = 1 - p.delta;

% labor supply
A1(6,3) = -1;
A1(6,7) = (1+p.kappa);
A2(6,11) = 1;

% Marginal utility of consumption (lambda)
A1(7,4) = p.OMEGA1;
A1(7,9) = p.OMEGA2;
A2(7,11) = 1;

% real return
A1(8,5) = (1-p.alpha)*p.alpha*p.YK_bar*p.eta*p.NL_bar;
A1(8,8) = p.theta; 
A2(8,1) = p.alpha*p.YK_bar*(1 + p.eta*p.NL_bar)*p.rho_z;
A2(8,11) = (1-p.alpha)*p.alpha*p.YK_bar;

% money stock
A1(9,9) = 1;
A2(9,2) = 1;
A2(9,9) = 1;
A2(9,12) = -1;

% money demand
A1(10,4) = -1;
A1(10,9) = 1;
A1(10,10) = (1/p.b)*(1/p.i_bar);

% Euler condition
A1(11,8) = 1;
A1(11,11) = 1;
A2(11,11) = 1;

% Fisher equation
A1(12,8) = 1;
A1(12,10) = -1;
A1(12,12) = 1;

A = inv(A1)*A2

% Variance-covariance matrix of shocks
SigmaXX = zeros(n1,n1);
SigmaXX(1,1) = p.sigma_z^2;
SigmaXX(2,2) = p.sigma_v^2;

st=1;

%% Solving the model
x10 = zeros(n1,1);                  %%initial state vector*/

% check eigenvalues
rtck = 0;
[Xi_eigvec,Xi_eigval] = eig(A);
[Xi_sortabs,Xi_sortindex] = sort(abs(diag(Xi_eigval)));
if Xi_sortabs(n1+1) < 1
    rtck = 1;
    disp('Not enough unstable roots')
else
    if Xi_sortabs(n1) > 1
        rtck = 1;
        disp('Not enough stable roots')
    end
end

%% Solving the model
if rtck == 0
    if sol == 'SHU'; % solution method via Shur decomposition
        tic
        [M,C] = basicsol(A,n1,n2,SigmaXX,x10,1.0);
        toc
        disp('Solution obtaining using basicsol.m')
        disp('M, where X(t+1) = M*X(t) + e(t+1) for predetermined variables')
        M
        disp('C, where x(t) = C*X(t) for forward-looking variables')
        C
    else
        if sol == 'LRP' % solution method using linear regular approach
            tic
            [M, C] = lrp(A,p.beta,n1,n2)
            toc
            disp('Solution obtained using olrp_tr.m')
            disp('M, where X(t+1) = M*X(t) + e(t+1) for predetermined variables')
            M
            disp('C, where x(t) = C*X(t) for forward-looking variables')
            C
        end
    end
end; % end of rtck loop

%% impulse responses
if imp == 1;
    disp('Impulse response functions')
else
        disp('Impulse reponses not chosen')
end
%% Constructing impulse responses
if imp == 1;
for sk = 1:2;  % loops over both shocks to construct impulse response functions
    if sk == 1;
        Shock0 = [1; zeros(n1-1,1)];   %%Impulse response wrt technology innovation*/
    else;
        Shock0 = [0; 1; zeros(n1-2,1)];   %%Impulse response wrt money innovation*/
    end

% Borrowed from Paul Soderlind's VarImpResp.m
x1 = zeros(Tbig,n1);   %%Simulate impulse response function*/
x1(:,:) = NaN;
x1M_t_1 = zeros(n1,1);           %%starting vector*/
t = 1;
  while t <= Tbig;
    if t == 1;
      Shock = Shock0;
    else;
      Shock = zeros(n1,1);
    end;
    x1M = M*x1M_t_1 + Shock;
    x1(t,:) = x1M';
    x1M_t_1 = x1M;
    t = t + 1;
  end;

%x1 = VarImpResp( M,Shock0,Tbig );  %VAR of x1(t)*/
    x2 = x1*C';                        %x2(t)*/
    x  = [x1 x2];                      %x(t) = [x1(t),x2(t)]*/
    
    tid = linspace(0,Tbig-1,Tbig-1)';
    tzero = tid - tid;
    % save results of impulse responses
    if sk == 1;
        z_output_b(:,ib) = x(st+1:Tbig,3);
        z_n_b(:,ib) = x(st+1:Tbig,7);
    else
        u_output_b(:,ib) = x(st+1:Tbig,3);
        u_n_b(:,ib) = x(st+1:Tbig,7);       
    end
end; % end of loop over shocks (sk)
end; % end of loop over imp
end; % end of loop over elements of b
u_output_b
u_n_b
figure
plot(tid,u_output_b(:,1),tid,u_output_b(:,2),':',tid,u_output_b(:,3),':',tid,u_output_b(:,4),'--')
legend('b=1','b=2','b=3','b=5')
figure
plot(tid,u_n_b(:,1),tid,u_n_b(:,2),':',tid,u_n_b(:,3),':',tid,u_n_b(:,4),'--')
legend('b=1','b=2','b=3','b=5')
